#include <stdio.h>
#include <string.h>
#include "aduc8xx.h"

#define ACK  (0x06)
#define NAK  (0x07)
#define DEFAULT_RECORD_LENGTH (0x10)


int checkSum(unsigned char *buf, int len)
{
  int i, crc;

  crc = 0;
  for (i = 0; i < len; i++) {
    crc += *(buf++);
  }
  crc &= 0xff;
  return ((256 - crc) & 0xff);
}

int identify(int fd)
{
  int idnum;
  char mes[26];
  int dmy;

  writeSerialPortFrom(fd, 4, "!Z\x00\xa6");
  if ((dmy = readSerialPortUpto(fd, mes, 25)) != 25) {
    fprintf(stderr, "Serial Port: Time Out.%d\n", dmy);
    return 0;
  }

  if (strncmp(mes, "ADI", 3) != 0) {
    mes[10] = '\0';
    printf("Not supported: Product id = %s\n", mes);
    return 0;
  }

  if ((idnum = identifyDevice(mes + 4)) == 0) {
    mes[8] = '\0';
    printf("%s is not supported.\n", mes);
    return 0;
  }

  mes[14] = '\0';
  if (mes[11] != '2') {
    printf("%s Loder is not supported.\n", mes + 10);
    return 0;
  }

  printf("OK: %s\n", mes);
  return idnum;
}


int writeBufferFrom(int fd, int command, int bufferSize, unsigned char *startPtr)
{
  char response;
  int i, j, isProg;
  unsigned char mess[DEFAULT_RECORD_LENGTH+9];

  mess[0] = 0x07;
  mess[1] = 0x0e;
  mess[2] = DEFAULT_RECORD_LENGTH + 4;
  mess[3] = command;
  mess[DEFAULT_RECORD_LENGTH+8] = '\0';
  for (i = 0; i < bufferSize; i += DEFAULT_RECORD_LENGTH) {
    mess[4] = (i >> 16) & 0xff;
    mess[5] = (i >> 8) & 0xff;
    mess[6] = i & 0xff;
    isProg = 0;
    for (j = 0; j < DEFAULT_RECORD_LENGTH; j++) {
      if ((mess[j+7] = startPtr[i+j]) != 0) {
	isProg = 1;
      }
    }
    if (isProg) {
      mess[DEFAULT_RECORD_LENGTH+7] = checkSum(mess+2, DEFAULT_RECORD_LENGTH+5);
      if (writeSerialPortFrom(fd, DEFAULT_RECORD_LENGTH+8, mess) != DEFAULT_RECORD_LENGTH+8) {
	printf("Serial write error.\n");
	return 0;
      }
      if (readSerialPortUpto(fd, &response, 1) != 1) {
	fprintf(stderr, "Serial Port: Time Out.\n");
	return 0;
      }
      if (response == ACK) {
	printf(".");
	fflush(stdout);
      } else if (response == NAK) {
	printf("NAK recieved.\n");
	return 0;
      } else {
	printf("Unknown response.\n");
	return 0;
      }
    }
  }
  return 1;
}


void writeProgramMemoryFrom(int fd, int idnum, unsigned char *startPtr)
{
  if (writeBufferFrom(fd, 'W', pmsize(idnum), startPtr) == 1) {
    printf("\nProgram memory written.\n");
  } else {
    printf("Fail to write program memory.\n");
  }
}

void writeDataMemoryFrom(int fd, int idnum, unsigned char *startPtr)
{
  if (writeBufferFrom(fd, 'E', dmsize(idnum), startPtr) == 1) {
    printf("\nData memory written.\n");
  } else {
    printf("Fail to write data memory.\n");
  }
}

void eraseProgramMemory(int fd)
{
  char response;

  if (writeSerialPortFrom(fd, 5, "\x07\x0e\001C\xbc") != 5) {
    fprintf(stderr, "Serial port wirte error.\n");
  }
  if (readSerialPortUpto(fd, &response, 1) != 1) {
    fprintf(stderr, "Serial Port: Time Out.\n");
    return;
  }
  if (response == ACK) {
    printf("Program memory erased.\n");
  } else if (response == NAK) {
    printf("Fail to erase program memory.\n");
  } else {
    printf("Unknown response.\n");
  }
}

void eraseProgramAndDataMemory(int fd)
{
  char response;

  writeSerialPortFrom(fd, 5, "\x07\x0e\001A\xbe");
  if (readSerialPortUpto(fd, &response, 1) != 1) {
    fprintf(stderr, "Serial Port: Time Out.\n");
    return;
  }
  if (response == ACK) {
    printf("Program and data memory erased.\n");
  } else if (response == NAK) {
    printf("Fail to erase program and data memory.\n");
  } else {
    printf("Unknown response.\n");
  }
}
